--
--
-- Uses Flex 10K20 Device
--
-- Jim Hamblen, Georgia Tech School of ECE
--
library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;


entity razzle is

Generic(ADDR_WIDTH: integer := 12; DATA_WIDTH: integer := 1);

   port( signal PB1, PB2, Clock : in std_logic;
        signal LSB_a, LSB_b, LSB_c, LSB_d, LSB_e, LSB_f, LSB_g, LSB_dp,
        MSB_a, MSB_b, MSB_c, MSB_d, MSB_e, MSB_f, MSB_g, MSB_dp : out std_logic;               
        signal Red,Green,Blue : out std_logic;
        signal Horiz_sync,Vert_sync : out std_logic;
        signal MOUSE_DATA: in std_logic;
        signal MOUSE_CLK : in std_logic;
        signal Flex_Switch_1, Flex_Switch_2, Flex_Switch_3, Flex_Switch_4: in std_logic;
        signal Flex_Switch_5, Flex_Switch_6, Flex_Switch_7, Flex_Switch_8: in std_logic);
       
		
		
end razzle;

architecture behavior of razzle is

-- Video Display Signals   
signal video_address: std_logic_vector(11 Downto 0);
signal H_count,V_count: std_logic_vector(9 Downto 0);
signal F_count: std_logic_vector(4 Downto 0);
signal Color_count: std_logic_vector(3 Downto 0);
signal Red_Data, Green_Data, Blue_Data, reset: std_logic;
signal sum, rowdist, coldist : std_logic_vector(10 DOWNTO 0);
signal nrowdist, ncoldist : std_logic_vector(10 DOWNTO 0);
signal slow_dither: std_logic_vector(3 DOWNTO 0);
signal dither: std_logic_vector(3 DOWNTO 0);
-- Signals for LED Display
signal LSB,MSB: std_logic_vector(3 Downto 0);
signal LSB_7SEG,MSB_7SEG: std_logic_vector(6 Downto 0);

-- Signals for Push buttons
signal PB1_sync, PB2_sync, PB2_Single_Pulse, PB1_Single_Pulse: std_logic; 
signal PB2_debounced, PB1_debounced, PB2_debounced_Sync, PB1_debounced_Sync: std_logic; 
signal PB1_debounced_delay, PB2_debounced_delay, Debounce_clock: std_logic;
signal SHIFT_PB1, SHIFT_PB2: std_logic_vector(3 Downto 0);

constant H_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(799,10); 
-- 799 is max horiz count
constant V_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(524,10); 
-- 524 is max vert count
signal video_on, video_on_H, video_on_V: std_logic;

begin           

-- Colors for pixel data on video signal
Red_Data <= sum(2);
Green_Data <= sum(3);
Blue_Data <= sum(4);


Red <=   Red_Data and video_on;
Green <= Green_Data and video_on;
Blue <=  Blue_Data and video_on;

-- video_on turns off pixel data when not in the view area
video_on <= video_on_H and video_on_V;


FRACTAL_COMPUTE: Process (reset, clock)
Begin
 If reset = '1' then
	dither <= "0000";
 Elsif (clock'event) and (clock='1') Then
 	IF video_on = '1'  THEN
		If V_count > 240 THEN
			rowdist <= V_count -240;
		else
			rowdist <= 240 - V_count;
		END IF;
		IF rowdist > 120 THEN
			nrowdist <= rowdist -120;
		else
			nrowdist <= 120 - rowdist;
		end if;	
		If H_count > 320 THEN
			coldist <= H_count -320;
		else
			coldist <= 320 - H_count;
		end if;
		if coldist >160 THEN
			ncoldist <= coldist -160;
		else
			ncoldist <= 160 - coldist;
		END IF;
-- sum assigns color to pixel based on row + col address	
	sum <= nrowdist + ncoldist + ( dither & "00") ;
	END If;
end if;
end process FRACTAL_COMPUTE;


process
begin
	wait until debounce_clock'event and debounce_clock = '1';
			dither <= dither + 1;
end process;

--Generate Horizontal and Vertical Timing Signals for Video Signal
VIDEO_DISPLAY: Process
Begin

Wait until(Clock'Event) and (Clock='1');
If Reset = '1' Then
 H_count <= CONV_STD_LOGIC_VECTOR(0,10);
 V_count <= CONV_STD_LOGIC_VECTOR(0,10);
 Video_on_H <= '0';
 Video_on_V <= '0';
Else
-- H_count counts pixels (640 + extra time for sync signals)
--
--   <-Clock out RGB Pixel Row Data ->   <-H Sync->
--   ------------------------------------__________--------
--   0                           640   659       755    799
--
If (H_count >= H_max) then
   H_count <= To_Stdlogicvector(B"0000000000");
Else
   H_count <= H_count + To_Stdlogicvector(B"0000000001");
End if;

--Generate Horizontal Sync Signal
If (H_count <= 755) and (H_count >= 659) Then
   Horiz_Sync <= '0';
ELSE
   Horiz_Sync <= '1';
End if;

--V_count counts rows of pixels (480 + extra time for sync signals)
--
--  <---- 480 Horizontal Syncs (pixel rows) -->  ->V Sync<-
--  -----------------------------------------------_______------------
--  0                                       480    493-494          524
--
If (V_count >= V_max) and (H_count >= 699) then
   V_count <= To_Stdlogicvector(B"0000000000");
Else If (H_count = 699) Then
   V_count <= V_count + To_Stdlogicvector(B"0000000001");
End if;
End if;

-- Generate Vertical Sync Signal
If (V_count <= 494) and (V_count >= 493) Then
   Vert_Sync <= '0';
   Debounce_CLock <= '0';
ELSE
   Vert_Sync <= '1';
   Debounce_Clock <= '1';
End if;



-- Generate Video on Screen Signals for Pixel Data
If (H_count <= 639) Then
   video_on_H <= '1';
ELSE
   video_on_H <= '0';
End if;


If (V_count <= 479) Then
   video_on_V <= '1';
ELSE
   video_on_V <= '0';
End if;


End if;

end process VIDEO_DISPLAY;

-- Connections to LED Display Segments
MSB_dp <= PB1_SYNC;
LSB_dp <= PB1_DEBOUNCED;
MSB <= dither(3 Downto 0);
LSB <= dither(3 Downto 0);

MSB_a <= NOT MSB_7SEG(6);
MSB_b <= NOT MSB_7SEG(5);
MSB_c <= NOT MSB_7SEG(4);
MSB_d <= NOT MSB_7SEG(3);
MSB_e <= NOT MSB_7SEG(2);
MSB_f <= NOT MSB_7SEG(1);
MSB_g <= NOT MSB_7SEG(0);

LSB_a <= NOT LSB_7SEG(6);
LSB_b <= NOT LSB_7SEG(5);
LSB_c <= NOT LSB_7SEG(4);
LSB_d <= NOT LSB_7SEG(3);
LSB_e <= NOT LSB_7SEG(2);
LSB_f <= NOT LSB_7SEG(1);
LSB_g <= NOT LSB_7SEG(0);

LED_DISPLAY: process  (MSB,LSB)
-- BCD to 7 Segment Decoders for LED Displays
begin
CASE MSB IS
        WHEN "0000" =>
            MSB_7SEG <= "1111110";
        WHEN "0001" =>
            MSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            MSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            MSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            MSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            MSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            MSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            MSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            MSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            MSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            MSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            MSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            MSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            MSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            MSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            MSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            MSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

CASE LSB IS
        WHEN "0000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111110");
        WHEN "0001" =>
            LSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            LSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            LSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            LSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            LSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            LSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            LSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            LSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            LSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            LSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            LSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            LSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            LSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            LSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            LSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

end process LED_DISPLAY;

RESET <= NOT PB2;

-- Sync extenal pushbutton inputs to chip clock
PUSH_BUTTON: process (clock)
begin
  wait until (clock'event) and (clock='1');
PB1_Sync <= NOT PB1;
PB2_Sync <= NOT PB2;
PB1_DEBOUNCED_SYNC <= PB1_DEBOUNCED;
PB2_DEBOUNCED_SYNC <= PB2_DEBOUNCED;
end process PUSH_BUTTON;

-- Debounce Button: Filters out mechanical bounce for around 80Ms.
-- Debounce clock uses Vert_Sync timing signal (16Ms) to save hardware
-- for clock prescaler
DEBOUNCE_BUTTON1: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB1(2 Downto 0) <= SHIFT_PB1(3 Downto 1);
  SHIFT_PB1(3) <= PB1_Sync;
  If SHIFT_PB1(3 Downto 0)="1111" THEN
    PB1_DEBOUNCED <= '1';
  ELSE 
    PB1_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON1;

DEBOUNCE_BUTTON2: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB2(2 Downto 0) <= SHIFT_PB2(3 Downto 1);
  SHIFT_PB2(3) <= PB2;
  If SHIFT_PB2(3 Downto 0)="1111" THEN
    PB2_DEBOUNCED <= '1';
  ELSE 
    PB2_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON2;

SINGLE_PULSE_PB1: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
  If RESET='1' Then
  PB1_SINGLE_PULSE <='0';
  PB1_DEBOUNCED_DELAY <= '1';
  ELSE
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
  IF PB1_DEBOUNCED_SYNC = '1' AND PB1_DEBOUNCED_DELAY = '0' THEN
   PB1_SINGLE_PULSE <= '1';
  ELSE
   PB1_SINGLE_PULSE <= '0';
  END IF;
  PB1_DEBOUNCED_DELAY <= PB1_DEBOUNCED_SYNC;
 End if;
end process SINGLE_PULSE_PB1;


SINGLE_PULSE_PB2: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
   If RESET='1' Then
    PB2_SINGLE_PULSE <='0';
    PB2_DEBOUNCED_DELAY <= '1';
   ELSE
    PB2_DEBOUNCED_DELAY <= PB2_DEBOUNCED_SYNC;
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
    IF PB2_DEBOUNCED_SYNC = '1' AND PB2_DEBOUNCED_DELAY = '0' THEN
     PB2_SINGLE_PULSE <= '1';
    ELSE
     PB2_SINGLE_PULSE <= '0';
    END IF;
  End if;
end process SINGLE_PULSE_PB2;


end behavior;

